import torch
import click
from pathlib import Path
import subprocess
from colorama import init, Fore, Back, Style
import shutil
import os, sys

# Initialize colorama (only needed once)
init(autoreset=True)

green = Fore.LIGHTGREEN_EX
red = Fore.LIGHTRED_EX
blue = Fore.LIGHTBLUE_EX
yellow = Fore.YELLOW
background_color = Back.BLACK  # You can choose different background colors
reset = Style.RESET_ALL

def get_folder_size(path):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(path):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            total_size += os.path.getsize(filepath)
    return total_size

def check_model_existence(repo_id, model_name):
    """
    Check if both the model and the tokenizer for the given model_name exist in the local cache.
    """
    model_path = Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_name}"
    return model_path.exists()

def list_models1():
    cache_dir = Path.home() / ".cache" / "huggingface" / "hub"
    click.echo("\n    Available Models: ")

    for model_dir in cache_dir.iterdir():
        if model_dir.is_dir() and model_dir.name.startswith("models--"):
            parts = model_dir.name.split("--")
            if len(parts) == 3:
                repo_id = parts[1]
                model_id = parts[2]

                # Check the size of the folder
                folder_size = get_folder_size(model_dir)
                if folder_size < 10 * 1024 * 1024:  # 10 MB in bytes
                    # Delete the folder if its size is less than 10MB
                    shutil.rmtree(model_dir)

                else:
                    click.echo(f"       * {blue}{repo_id}/{model_id}{reset}")


def check_nvidia_gpu():
    # Check if CUDA (Nvidia GPU) is available
    if torch.cuda.is_available():
        # Get the total and available memory
        total_memory = torch.cuda.get_device_properties(0).total_memory
        available_memory = total_memory - torch.cuda.memory_allocated(0)

        # Convert bytes to GB for easier reading
        total_memory_gb = total_memory / (1024 ** 3)
        available_memory_gb = available_memory / (1024 ** 3)

        return True, total_memory_gb, available_memory_gb
    else:
        return False, None, None

def get_model_size(repo_id, model_name):
    model_path = Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_name}"
    try:
        du_output = subprocess.check_output(['du', '-sb', model_path], universal_newlines=True)
        size_str = du_output.split()[0]
        size_in_bytes = int(size_str)
        size_in_gb = size_in_bytes / (1024 ** 3)  # Convert bytes to GB
        return size_in_gb
    except subprocess.CalledProcessError:
        return 0

def show_hardware(model_name):
    click.echo(f"\n    Checking hardware compatibility...\n")

    # Check if the model is downloaded
    try:
        repo_id, model_name = model_name.split('/')
    except ValueError:
        click.echo(f"\n    {red}[Invalid]{reset} Model format\n    Use format: {blue}repo_id/model_name{reset}\n")
        sys.exit()

    if not check_model_existence(repo_id, model_name):
        click.echo(f"    {red}[Invalid]{reset} model: {blue}{repo_id}/{model_name}{reset}")
        f"{list_models1()}\n"
        return
    else:
        # Check Nvidia GPU status and memory
        gpu_available, total_memory_gb, available_memory_gb = check_nvidia_gpu()

        if gpu_available:
            model_size = get_model_size(repo_id, model_name)
            if model_size <= total_memory_gb:
                click.echo(f"    {green}[Valid]{reset} GPU Memory for {blue}{model_name}{reset}")
                click.echo(f"       * GPU memory: {total_memory_gb:.2f} GB")
                click.echo(f"       * Model size: {model_size:.2f} GB\n")
            else:
                click.echo(f"    {red}[Invalid]{reset} GPU Memory for {blue}{model_name}{reset}")
                click.echo(f"       * GPU memory: {total_memory_gb:.2f} GB")
                click.echo(f"       * Model size: {model_size:.2f} GB\n")
        else:
            click.echo(f"    {red}[Invalid]{reset} CUDA drivers\n    Check CUDA Drivers ${yellow}nvidia-smi{reset}\n\n    You can still execute locally using {yellow}CPU{reset}\n")



import click
import shutil
from pathlib import Path
import os
from colorama import init, Fore, Style, Back

init(autoreset=True)

green = Fore.LIGHTGREEN_EX
red = Fore.LIGHTRED_EX
blue = Fore.LIGHTBLUE_EX
yellow = Fore.YELLOW
background_color = Back.BLACK  # You can choose different background colors
reset = Style.RESET_ALL


def list_models():
    cache_dir = Path.home() / ".cache" / "huggingface" / "hub"

    models_found = False  # Flag to track if models are found

    click.echo("\n    Available Models: ")

    for model_dir in cache_dir.iterdir():
        if model_dir.is_dir() and model_dir.name.startswith("models--"):
            parts = model_dir.name.split("--")
            if len(parts) == 3:
                repo_id = parts[1]
                model_id = parts[2]

                # Check the size of the folder
                folder_size = get_folder_size(model_dir)
                if folder_size < 500 * 1024 * 1024:  # 10 MB in bytes
                    # Delete the folder if its size is less than 10MB
                    shutil.rmtree(model_dir)
                else:
                    click.echo(f"       * {blue}{repo_id}/{model_id}{reset}")
                    models_found = True  # Set the flag to True

    if not models_found:
        click.echo(f"       {red}!!! No models available.{reset}")

    click.echo(f"\n    Download new models ${yellow}lc add --model <model_name>{reset}\n")


def get_folder_size(path):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(path):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            total_size += os.path.getsize(filepath)
    return total_size



import click
import os

script_directory = os.path.dirname(os.path.abspath(__file__))
log_file_path = os.path.join(script_directory, 'execution_logs.txt')
def fetch_result():
    if os.path.exists(log_file_path):
        click.echo(f"\n    Fetching previous LLM sessions...\n")
        with open(log_file_path, 'r') as file:
            content = file.read()
            click.echo(content)
    else:
        click.echo("\n    No previous logs!\n")


import click
from transformers import AutoTokenizer, AutoModelForCausalLM
from pathlib import Path
from colorama import init, Fore, Back, Style
import shutil
import os

init(autoreset=True)
green = Fore.LIGHTGREEN_EX
red = Fore.LIGHTRED_EX
blue = Fore.LIGHTBLUE_EX
yellow = Fore.YELLOW
background_color = Back.BLACK  # You can choose different background colors
reset = Style.RESET_ALL


def get_folder_size(path):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(path):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            total_size += os.path.getsize(filepath)
    return total_size

def check_model_existence(repo_id, model_name):
    """
    Check if both the model and the tokenizer for the given model_name exist in the local cache.
    """
    model_path = Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_name}"
    return model_path.exists()

def download_model(repo_id, model_name):
    """
    Download the model and the tokenizer for the given model_name.
    """
    click.echo(f"\n    Downloading {blue}{repo_id}/{model_name}{reset} from {blue}HuggingFace{reset}...\n")
    AutoModelForCausalLM.from_pretrained(f"{repo_id}/{model_name}")
    AutoTokenizer.from_pretrained(f"{repo_id}/{model_name}")
    click.echo(f"\n    {blue}{repo_id}/{model_name}{reset} downloaded successfully!\n")

def list_models():
    cache_dir = Path.home() / ".cache" / "huggingface" / "hub"

    click.echo("\n    Available Models: ")

    for model_dir in cache_dir.iterdir():
        if model_dir.is_dir() and model_dir.name.startswith("models--"):
            parts = model_dir.name.split("--")
            if len(parts) == 3:
                repo_id = parts[1]
                model_id = parts[2]

                # Check the size of the folder
                folder_size = get_folder_size(model_dir)
                if folder_size < 50 * 1024 * 1024:  # 500 MB in bytes
                    # Delete the folder if its size is less than 10MB
                    shutil.rmtree(model_dir)

                else:
                    click.echo(f"       * {blue}{repo_id}/{model_id}{reset}")

    click.echo(f"\n    Download new models ${yellow}lc add --model <model_name>{reset}\n")

def add_model(model_identifier):
    # Splitting the identifier into repo_id and model_name
    try:
        repo_id, model_name = model_identifier.split('/')
        if check_model_existence(repo_id, model_name):
            click.echo(f"\n    Model {blue}{repo_id}/{model_name}{reset} is already available!")
            list_models()
        else:
            try:
                download_model(repo_id, model_name)
                list_models()
            except OSError:
                click.echo(f"    {blue}{model_identifier}{reset} is not a valid model identifier listed on 'https://huggingface.co/models'\n")
                shutil.rmtree(Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_name}")

    except ValueError:
        click.echo(f"\n    {red}[Invalid]{reset} Model format\n    Use format: {blue}repo_id/model_name{reset}\n")


import click
from colorama import init, Fore, Back, Style

init(autoreset=True)
green = Fore.LIGHTGREEN_EX
red = Fore.LIGHTRED_EX
blue = Fore.LIGHTBLUE_EX
yellow = Fore.YELLOW
background_color = Back.BLACK  # You can choose different background colors
reset = Style.RESET_ALL

def list_tasks():
    tasks = f"""
    Available Tasks:
       * {blue}NER{reset}                 #Name Entity Recognition from input content
       * {blue}Summary{reset}             #Summary of input content
       * {blue}AnalyseSentiment{reset}    #Sentiment analysis of input content
       * {blue}DetectBias{reset}          #Bias Detection in input content
       * {blue}TagTopic{reset}            #Topic tagging to input content
       * {blue}Custom{reset}              #Custom user prompt
    """
    click.echo(tasks)


import sys
import click
from pathlib import Path
from transformers import AutoTokenizer, AutoModelForCausalLM, pipeline
import torch
import shutil
from ..config import (available_tasks)

device = "cuda" if torch.cuda.is_available() else "cpu"
import os
from datetime import datetime
from colorama import init, Fore, Style, Back

init(autoreset=True)

green = Fore.LIGHTGREEN_EX
red = Fore.LIGHTRED_EX
blue = Fore.LIGHTBLUE_EX
magenta = Fore.LIGHTMAGENTA_EX
yellow = Fore.YELLOW
background_color = Back.BLACK  # You can choose different background colors
reset = Style.RESET_ALL

# Setting up logger
script_directory = os.path.dirname(os.path.abspath(__file__))
log_file_path = os.path.join(script_directory, 'execution_logs.txt')


def get_folder_size(path):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(path):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            total_size += os.path.getsize(filepath)
    return total_size


def list_tasks1():
    tasks = f"""
    Available tasks:
       * {blue}NER{reset}                 #Name Entity Recognition from input content
       * {blue}Summary{reset}             #Summary of input content
       * {blue}AnalyseSentiment{reset}    #Sentiment analysis of input content
       * {blue}DetectBias{reset}          #Bias Detection in input content
       * {blue}TagTopic{reset}            #Topic tagging to input content
       * {blue}Custom{reset}              #Custom user prompt
    """
    click.echo(tasks)


# Function to check if the model exists in the local cache
def check_model_existence(repo_id, model_name):
    model_path = Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_name}"
    return model_path.exists()


# Function to list available models
def list_models1():
    cache_dir = Path.home() / ".cache" / "huggingface" / "hub"
    click.echo("\n    Available Models: ")

    for model_dir in cache_dir.iterdir():
        if model_dir.is_dir() and model_dir.name.startswith("models--"):
            parts = model_dir.name.split("--")
            if len(parts) == 3:
                repo_id = parts[1]
                model_id = parts[2]

                # Check the size of the folder
                folder_size = get_folder_size(model_dir)
                if folder_size < 50 * 1024 * 1024:  # 10 MB in bytes
                    # Delete the folder if its size is less than 10MB
                    shutil.rmtree(model_dir)

                else:
                    click.echo(f"       * {blue}{repo_id}/{model_id}{reset}")


def execute(task, model, input_text=None):

    lowercase_tasks = [task.lower() for task in available_tasks]
    if task.lower() not in lowercase_tasks:
        click.echo(f"\n    {red}!!!{reset} Task {blue}{task}{reset} is not available.")
        f"{list_tasks1()}\n"
        return

    repo_id, model_id = model.split('/') if '/' in model else (None, model)
    model_path = Path.home() / ".cache" / "huggingface" / "hub" / f"models--{repo_id.replace('/', '--')}--{model_id}"
    if not model_path.exists():
        click.echo(
            f"\n    {red}!!!{reset} Model {blue}{model}{reset} is not available.\n    Add model using ${yellow}lc add --model {model}{reset}")
        f"{list_models1()}\n"
        return

    model_path = f"{repo_id}/{model_id}"
    tokenizer = AutoTokenizer.from_pretrained(model_path)
    model = AutoModelForCausalLM.from_pretrained(model_path, torch_dtype=torch.float16, device_map="auto",
                                                 trust_remote_code=True)

    while True:
        click.echo(f"\n    Executing task {blue}{task}{reset} using model {blue}{repo_id}/{model_id}{reset}...\n")
        lowercase_tasks = [task.lower() for task in available_tasks]
        if task.lower() not in lowercase_tasks:
            click.echo(f"    {red}!!!{reset} Task {blue}{task}{reset} is not available.")
            f"{list_tasks1()}\n"
            task = input("\n    Enter task: ")

        if check_model_existence(repo_id, model_id):
            if task.lower() == 'custom':
                # For 'Custom' task, prompt for custom input if none is provided
                custom_input = input_text if input_text is not None else input(
                    f"    Enter your {blue}custom{reset} prompt: ")
                prompt = f"[INST] {custom_input} [/INST]"

            elif task.lower() == 'ner':
                ner_text = input_text if input_text is not None else input(
                    f"    {magenta}Enter text to recognise entities{reset}\n")
                prompt = f"[INST] Recognise the entities from the following text: {ner_text}. [/INST]"

            elif task.lower() == 'tagtopic':
                topics = input(f"    {magenta}Enter list of topics e.g. technology, art, news etc..{reset}\n")
                tagtopic_text = input(f"    {magenta}Enter input text for topic tagging:{reset}\n")
                prompt = f"[INST] Classify the following text in one of the topics from the list [{topics}]: {tagtopic_text}. Strictly respond with a one-word answer only. [/INST]"

            elif task.lower() == 'summary':
                summary_text = input_text if input_text is not None else input(
                    f"    {magenta}Enter text to summarise{reset}\n")
                prompt = f"[INST] Concisely summarise the following text: {summary_text}. [/INST]"

            elif task.lower() == 'analysesentiment':
                sentiment_text = input_text if input_text is not None else input(
                    f"    {magenta}Enter text to analyse sentiment{reset}\n")
                prompt = f"[INST] Analyze the sentiment of the following text: {sentiment_text}. [/INST]"

            elif task.lower() == 'detectbias':
                bias_text = input_text if input_text is not None else input(
                    f"    {magenta}Enter text to detect bias{reset}\n")
                prompt = f"[INST] Analyze the following article for any biases: {bias_text}. [/INST]"

            def generate_text():
                time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')

                pipe = pipeline(task="text-generation", model=model, tokenizer=tokenizer, max_length=2800)
                result = pipe(prompt, pad_token_id=pipe.tokenizer.eos_token_id)
                generated_text = result[0]['generated_text'].replace(prompt, "").strip()
                print(f"\n    {yellow}Generated text:{reset} {generated_text}")
                if task.lower() == 'custom':
                    log_entry = f"\n###################################################\n\n    {magenta}[{time}]{reset}\n    {magenta}Model:{reset} {model_id}\n    {magenta}Task:{reset} {task}\n    {magenta}Input:{reset} {custom_input}\n    {magenta}Output:{reset} {generated_text}\n"
                elif task.lower() == 'ner':
                    log_entry = f"\n###################################################\n\n    {magenta}[{time}]{reset}\n    {magenta}Model:{reset} {model_id}\n    {magenta}Task:{reset} {task}\n    {magenta}Input:{reset} {ner_text}\n    {magenta}Output:{reset} {generated_text}\n"
                elif task.lower() == 'tagtopic':
                    log_entry = f"\n###################################################\n\n    {magenta}[{time}]{reset}\n    {magenta}Model:{reset} {model_id}\n    {magenta}Task:{reset} {task}\n    {magenta}Input:{reset} {tagtopic_text}\n    {magenta}Output:{reset} {generated_text}\n"
                else:
                    log_entry = f"\n###################################################\n\n    {magenta}[{time}]{reset}\n    {magenta}Model:{reset} {model_id}\n    {magenta}Task:{reset} {task}\n    {magenta}Input:{reset} {input_text}\n    {magenta}Output:{reset} {generated_text}\n"

                if not os.path.exists(log_file_path):
                    with open(log_file_path, 'w'):
                        pass
                with open(log_file_path, 'a') as log_file:
                    log_file.write(log_entry)

            generate_text()
        else:
            click.echo(
                f"    {magenta}Model{reset} {blue}{model}{reset} {magenta}is not downloaded. You can download it using{reset} ${yellow}lc add --model {model}{reset}")

        torch.cuda.empty_cache()
        while True:
            continue_choice = input(f"\n    {magenta}Do you want to exit? (yes/no):{reset} ").strip().lower()
            if continue_choice == 'yes' or continue_choice == 'y':
                sys.exit()
            elif continue_choice == 'no' or continue_choice == 'n':
                task = input("\n    Enter task: ")
                input_text = None  # Reset input_text for next iteration
                break
            else:
                print(f"    {red}\n    Please select yes/y or no/n{reset}")



from setuptools import setup, find_packages
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text(encoding="utf-8")

setup(
    name='llm_connect',
    version='1.0.1',
    packages=find_packages(),
    install_requires=['click==8.1.7', 'setuptools~=68.2.0', 'transformers~=4.34.0','torch~=2.1.0','accelerate', 'bitsandbytes','colorama'],
    description="LLM Connect API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="CeADAR Connect Group",
    entry_points={
    'console_scripts': [
        'lc = llm_connect_api:cli',
    ],
},
 
)




import click
from llm_connect_api.commands.add_model import add_model
from llm_connect_api.commands.fetch_result import fetch_result
from llm_connect_api.commands.hardware import show_hardware
from llm_connect_api.commands.list_models import list_models
from llm_connect_api.commands.list_tasks import list_tasks
from llm_connect_api.commands.execute import *
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
import textwrap

@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """
    LLMConnect API is a developer-friendly Python-based CLI utility designed to manage and execute Language Models from huggingface on local servers or clusters. It enables users to run a variety of standard and custom tasks with popular models such as Llama-2, Mistral, Falcon, etc., and also supports multiple features including integration of new LLMs, hardware compatability check, and logging functionality.

    Available command:

        * lc list models

        * lc list tasks

        * lc fetch

        * lc add --model <MODEL_NAME>

        * lc hardware --model <MODEL_NAME>

        * lc exec --task <TASK_NAME> --model <MODEL_NAME> --input <INPUT>
    """
    pass

@cli.group()
def list():
    """
    List all available tasks or models."""
    pass

@list.command(name="models", help="List available models",context_settings=CONTEXT_SETTINGS)
def models():
    list_models()

@list.command(name="tasks", help="List available tasks",context_settings=CONTEXT_SETTINGS)
def tasks():
    list_tasks()

@cli.command(context_settings=CONTEXT_SETTINGS)
@click.option('--model', required=True)
def add(model):
    """Add new Hugging Face Model.

    * Model format: repo_id/model_id"""
    add_model(model)

@cli.command(context_settings=CONTEXT_SETTINGS)
def fetch():
    """Fetch the logs of previous sessions."""
    fetch_result()

@cli.command(context_settings=CONTEXT_SETTINGS)
@click.option('--model', required=True,
              help='Model name in format: repoID/modelID')

def hardware(model):
    """Check hardware compatibility for given Hugging Face model.

    * Model format: repo_id/model_id."""
    show_hardware(model)

# @click.option('--task', required=True, help='task to execute')
# @click.option('--model', required=True, help='model to execute')
# @click.option('--input', required=True, help='input prompt')

# @cli.command()
# @click.option("--task", required=True, type=click.Choice(available_tasks, case_sensitive=False), help="Specify the task name")
# @click.option('--model', required=True, help='model to execute')
# @click.option("--input", default=None, help="Specify input text (optional)")

@cli.command(context_settings=CONTEXT_SETTINGS)
@click.option("--task", required=True, type=click.STRING, help="Specify the task name")
@click.option("--model", required=True, type=click.STRING, help="Specify the model name (repoID/modelID)")
@click.option("--input", required=False,  type=click.STRING, help="Specify input text (optional)")
def exec(task, model, input):
    """Execute an input prompt with given model and given task."""
    execute(task, model, input)


if __name__ == '__main__':
    cli()